package cc.smtweb.system.bpm.web.engine.dynPage;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.mvc.service.AbstractHandler;
import cc.smtweb.framework.core.mvc.service.SwListData;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.system.bpm.web.design.form.ModelFormHelper;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.design.form.define.PageDatasets;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by Akmm at 2022/5/26 17:45
 */
public class DynPageHandler extends AbstractHandler {
    //页面定义id
    protected long pageId;
    protected PageDatasets datasets;

    protected DynPageProvider provider = new DynPageProvider();

    @Override
    public void init(SwMap params, UserSession us) {
        super.init(params, us);
        pageId = params.readLong("pageId");

        datasets = ModelFormHelper.parsePageDataset(pageId);
        if (datasets == null || datasets.list == null) throw new SwException("没有找到页面定义数据！");

        provider.pageId = pageId;
        provider.datasets = datasets;
    }

    protected PageDataset findDataset(String name) {
        return datasets.findByName(name);
    }

    //获取第一个，主数据集
    protected PageDataset findMasterDataset() {
        return datasets.list.get(0);
    }

    //新增操作，初始化定义的数据集
    public R add() {
        //返回的数据，以dataset.name为key，查出的结果（bean或list）为value
        SwMap mapRet = new SwMap();
        for (PageDataset dataSet : datasets.list) {
            if (!dataSet.canEdit) {
                continue;
            }
            if (SwEnum.DatasetType.FORM.value.equals(dataSet.type)) {//单表
                //懒加载，给个空对象
                if (dataSet.lazy) {
                    mapRet.put(dataSet.name, new SwMap());
                    continue;
                }
                mapRet.put(dataSet.name, DynPageHelper.createBean(dataSet));
            } else if (SwEnum.DatasetType.TREE.value.equals(dataSet.type)) {//树
                mapRet.put(dataSet.name, new SwMap());
            } else if (!SwEnum.DatasetType.ENUM.value.equals(dataSet.type)) {//非枚举
                mapRet.put(dataSet.name, SwListData.create(null, 0));
            }
        }
        return R.success(mapRet);
    }

    public R loadOne() {
    //数据集
        String dbName = params.readString("dataset");
        //过滤条件
        SwMap filter = (SwMap)params.get("filter");
        //对应的数据集定义
        PageDataset pageDataSet = findDataset(dbName);
        if (pageDataSet == null) throw new SwException("没有找到指定的的数据集定义：" + dbName + "！");

        DynRetBean bean = null;
        if (SwEnum.DatasetType.LIST.value.equals(pageDataSet.type)) {//列表类
            bean = DynRetBean.createList(provider.loadList(filter, pageDataSet));
        } else if (SwEnum.DatasetType.TREE.value.equals(pageDataSet.type)) {//树类
//            return new DynRetBean(loadTree());
        } else {//列表类
            bean = DynRetBean.createBean(provider.loadData(filter, pageDataSet));
        }

        return R.success(bean);
    }
}
